@@ -6,11 +6,10 @@
     Provides OAuth 1.0/a, 2.0 and Ofly service containers.
 '''
 
+from rauth.compat import urlencode
 from rauth.session import OAuth1Session, OAuth2Session, OflySession
 from rauth.utils import ENTITY_METHODS, parse_utf8_qsl
 
-from urllib import quote, urlencode
-
 PROCESS_TOKEN_ERROR = ('Decoder failed to handle {key} with data as returned '
                        'by provider. A different decoder may be needed. '
                        'Provider returned: {raw}')
@@ -20,12 +19,14 @@ def process_token_request(r, decoder, *args):
     try:
         data = decoder(r.content)
         return tuple(data[key] for key in args)
-    except KeyError, e:  # pragma: no cover
+    except KeyError as e:  # pragma: no cover
         bad_key = e.args[0]
         raise KeyError(PROCESS_TOKEN_ERROR.format(key=bad_key, raw=r.content))
 
 
 class Service(object):
+    __attrs__ = ['name', 'base_url', 'authorize_url']
+
     def __init__(self, name, base_url, authorize_url):
         #: The service name, e.g. 'twitter'.
         self.name = name
@@ -36,6 +37,14 @@ class Service(object):
         #: The authorization URL.
         self.authorize_url = authorize_url
 
+    def __getstate__(self):
+        return dict((attr, getattr(self, attr, None)) for
+                    attr in self.__attrs__)
+
+    def __setstate__(self, state):
+        for attr, value in state.items():
+            setattr(self, attr, value)
+
 
 class OAuth1Service(Service):
     '''
@@ -112,7 +121,16 @@ class OAuth1Service(Service):
     :param session_obj: Object used to construct sessions with, defaults to
         :class:`rauth.OAuth1Session <OAuth1Session>`
     :type session_obj: :class:`Session`
+    :param signature_obj: Object used to construct signatures with, defaults
+         to :class:`rauth.HmacSha1Signature <HmacSha1Signature>`
+    :type signature_obj: :class:`SignatureMethod`
     '''
+    __attrs__ = Service.__attrs__ + ['consumer_key',
+                                     'consumer_secret',
+                                     'request_token_url',
+                                     'access_token_url',
+                                     'session_obj']
+
     def __init__(self,
                  consumer_key,
                  consumer_secret,
@@ -121,7 +139,8 @@ class OAuth1Service(Service):
                  access_token_url=None,
                  authorize_url=None,
                  base_url=None,
-                 session_obj=None):
+                 session_obj=None,
+                 signature_obj=None):
 
         #: Client credentials.
         self.consumer_key = consumer_key
@@ -134,6 +153,9 @@ class OAuth1Service(Service):
         #: Object used to construct sessions with.
         self.session_obj = session_obj or OAuth1Session
 
+        #: Object used to construct signatures with.
+        self.signature_obj = signature_obj
+
         super(OAuth1Service, self).__init__(name,
                                             base_url,
                                             authorize_url)
@@ -156,12 +178,13 @@ class OAuth1Service(Service):
                                        self.consumer_secret,
                                        access_token,
                                        access_token_secret,
-                                       signature,
+                                       signature or self.signature_obj,
                                        service=self)
         else:  # pragma: no cover
             session = self.session_obj(self.consumer_key,
                                        self.consumer_secret,
-                                       signature=signature,
+                                       signature=signature
+                                       or self.signature_obj,
                                        service=self)
         return session
 
@@ -225,7 +248,7 @@ class OAuth1Service(Service):
             request querystring.
         :type \*\*params: dict
         '''
-        params.update({'oauth_token': quote(request_token)})
+        params.update({'oauth_token': request_token})
         return self.authorize_url + '?' + urlencode(params)
 
     def get_raw_access_token(self,
@@ -388,6 +411,11 @@ class OAuth2Service(Service):
         :class:`OAuth2Session`
     :type session_obj: :class:`rauth.Session`
     '''
+    __attrs__ = Service.__attrs__ + ['client_id',
+                                     'client_secret',
+                                     'access_token_url',
+                                     'session_obj']
+
     def __init__(self,
                  client_id,
                  client_secret,
@@ -549,6 +577,11 @@ class OflyService(Service):
         `rauth.OflySession`
     :type session_obj: :class:`rauth.Session`
     '''
+    __attrs__ = Service.__attrs__ + ['app_id',
+                                     'app_secret',
+                                     'user_id',
+                                     'session_obj']
+
     def __init__(self,
                  app_id,
                  app_secret,
