@@ -6,13 +6,13 @@
     Specially wrapped Requests' :class:`~request.sessions.Session` objects.
 '''
 
+from copy import deepcopy
 from datetime import datetime
 from hashlib import sha1, md5
 from random import random
 from time import time
-from urllib import quote
-from urlparse import parse_qsl, urljoin, urlsplit
 
+from rauth.compat import quote, parse_qsl, urljoin, urlsplit, is_basestring
 from rauth.oauth import HmacSha1Signature
 from rauth.utils import (absolute_url, CaseInsensitiveDict, ENTITY_METHODS,
                          FORM_URLENCODED, get_sorted_params,
@@ -24,6 +24,8 @@ OAUTH1_DEFAULT_TIMEOUT = OAUTH2_DEFAULT_TIMEOUT = OFLY_DEFAULT_TIMEOUT = 300.0
 
 
 class RauthSession(Session):
+    __attrs__ = Session.__attrs__ + ['service']
+
     def __init__(self, service):
         #: A back reference to a service wrapper, if we're using one.
         self.service = service
@@ -57,7 +59,7 @@ class OAuth1Session(RauthSession):
         session = OAuth1Session('123',
                                 '456',
                                 access_token='321',
-                                access_token_secret'654')
+                                access_token_secret='654')
 
     You now have a session object which can be used to make requests exactly as
     you would with a normal Requests' :class:`~requests.sessions.Session`
@@ -87,6 +89,12 @@ class OAuth1Session(RauthSession):
         `None`.
     :type service: :class:`rauth.Service`
     '''
+    __attrs__ = RauthSession.__attrs__ + ['consumer_key',
+                                          'consumer_secret',
+                                          'access_token',
+                                          'access_token_secret',
+                                          'signature']
+
     VERSION = '1.0'
 
     def __init__(self,
@@ -145,16 +153,21 @@ class OAuth1Session(RauthSession):
             req_kwargs['headers'].get('Content-Type') == FORM_URLENCODED
 
         # inline string conversion
-        if isinstance(req_kwargs.get('params'), basestring):
+        if is_basestring(req_kwargs.get('params')):
             req_kwargs['params'] = dict(parse_qsl(req_kwargs['params']))
 
-        if isinstance(req_kwargs.get('data'), basestring) and form_urlencoded:
+        if is_basestring(req_kwargs.get('data')) and form_urlencoded:
             req_kwargs['data'] = dict(parse_qsl(req_kwargs['data']))
 
         req_kwargs.setdefault('timeout', OAUTH1_DEFAULT_TIMEOUT)
 
         oauth_params = self._get_oauth_params(req_kwargs)
 
+        # ensure we always create new instances of dictionary elements
+        for key, value in req_kwargs.items():
+            if isinstance(value, dict):
+                req_kwargs[key] = deepcopy(value)
+
         # sign the request
         oauth_params['oauth_signature'] = \
             self.signature.sign(self.consumer_secret,
@@ -224,7 +237,8 @@ class OAuth1Session(RauthSession):
         oauth_params = {}
 
         oauth_params['oauth_consumer_key'] = self.consumer_key
-        oauth_params['oauth_nonce'] = sha1(str(random())).hexdigest()
+        oauth_params['oauth_nonce'] = sha1(
+            str(random()).encode('ascii')).hexdigest()
         oauth_params['oauth_signature_method'] = self.signature.NAME
         oauth_params['oauth_timestamp'] = int(time())
 
@@ -292,6 +306,10 @@ class OAuth2Session(RauthSession):
         `None`.
     :type service: :class:`rauth.Service`
     '''
+    __attrs__ = RauthSession.__attrs__ + ['client_id',
+                                          'client_secret',
+                                          'access_token']
+
     def __init__(self,
                  client_id,
                  client_secret,
@@ -326,10 +344,10 @@ class OAuth2Session(RauthSession):
 
         url = self._set_url(url)
 
-        if isinstance(req_kwargs['params'], basestring):
+        if is_basestring(req_kwargs['params']):
             req_kwargs['params'] = dict(parse_qsl(req_kwargs['params']))
 
-        if bearer_auth:
+        if bearer_auth and self.access_token is not None:
             bearer_token = 'Bearer {token}'.format(token=self.access_token)
             bearer_header = {'Authorization': bearer_token}
             req_kwargs.setdefault('headers', {})
@@ -381,6 +399,10 @@ class OflySession(RauthSession):
         `None`.
     :type service: :class:`rauth.Service`
     '''
+    __attrs__ = RauthSession.__attrs__ + ['app_id',
+                                          'app_secret',
+                                          'user_id']
+
     def __init__(self,
                  app_id,
                  app_secret,
@@ -427,7 +449,7 @@ class OflySession(RauthSession):
         assert user_id is not None, \
             'An oflyUserid must be provided as `user_id`.'
 
-        if isinstance(req_kwargs['params'], basestring):
+        if is_basestring(req_kwargs['params']):
             req_kwargs['params'] = dict(parse_qsl(req_kwargs['params']))
 
         req_kwargs['params'].update({'oflyUserid': user_id})
@@ -441,7 +463,7 @@ class OflySession(RauthSession):
         # NOTE: Requests can't seem to handle unicode objects, instead we can
         # encode a string here.
         req_kwargs['params'] = params
-        if isinstance(req_kwargs['params'], unicode):
+        if not isinstance(req_kwargs['params'], bytes):
             req_kwargs['params'] = req_kwargs['params'].encode('utf-8')
 
         return super(OflySession, self).request(method, url, **req_kwargs)
@@ -470,7 +492,7 @@ class OflySession(RauthSession):
             raise TypeError('hash_meth must be one of "sha1", "md5"')
 
         now = datetime.utcnow()
-        milliseconds = now.microsecond / 1000
+        milliseconds = now.microsecond // 1000
 
         time_format = '%Y-%m-%dT%H:%M:%S.{0}Z'.format(milliseconds)
         ofly_params = {'oflyAppId': app_id,
@@ -484,9 +506,12 @@ class OflySession(RauthSession):
             signature_base_string += get_sorted_params(params) + '&'
         signature_base_string += get_sorted_params(ofly_params)
 
+        if not isinstance(signature_base_string, bytes):
+            signature_base_string = signature_base_string.encode('utf-8')
+
         ofly_params['oflyApiSig'] = \
             hash_meth(signature_base_string).hexdigest()
 
-        all_params = dict(ofly_params.items() + params.items())
+        all_params = dict(tuple(ofly_params.items()) + tuple(params.items()))
 
         return get_sorted_params(all_params)
